@@ -256,11 +256,23 @@
         },
 
         loadToolbar: function () {
-            CMS.API.StructureBoard._loadToolbar()
-                .done(function (newToolbar) {
-                    CMS.API.Toolbar._refreshMarkup($(newToolbar).find('.cms-toolbar'));
-                })
-                .fail(CMS.API.Helpers.reloadBrowser);
+            if (CMS.settings && CMS.settings.version && this._toolbar_bug_version(CMS.settings.version)) {
+                // Before django CMS 3.10 a bug prevents the toolbar to be loaded correctly
+                // Refresh whole page instead
+                CMS.API.Helpers.reloadBrowser();
+            } else {
+                CMS.API.StructureBoard._loadToolbar()
+                    .done(function (newToolbar) {
+                        CMS.API.Toolbar._refreshMarkup($(newToolbar).find('.cms-toolbar'));
+                    })
+                    .fail(CMS.API.Helpers.reloadBrowser);
+            }
+        },
+
+        _toolbar_bug_version: function (version) {
+            var parts = version.split('.');
+
+            return parts[0] === '3' && parts[1].length < 2;
         },
 
         storeCSSlinks: function () {
@@ -463,6 +475,9 @@
             CMS.CKEditor.touchdevice = 'ontouchstart' in window || navigator.msMaxTouchPoints;  // on touch device?
             if (!CMS.CKEditor.touchdevice) {  // no inline editing on touch devices to not interfere with scrolling
                 CMS.CKEditor.initInlineEditors();
+                $('div.cms a.cms-btn.cms-edit-toggle').show();
+            } else {
+                $('div.cms a.cms-btn.cms-edit-toggle').hide();
             }
             CMS.CKEditor.initAdminEditors();
         },
