@@ -1,8 +1,5 @@
-# -*- coding: utf-8 -*-
 from django import template
 
-from djangocms_text_ckeditor.compat import LTE_CMS_3_3
-
 
 register = template.Library()
 
@@ -11,9 +8,6 @@ register = template.Library()
 def render_plugin_preview(context, plugin):
     request = context['request']
 
-    if LTE_CMS_3_3:
-        return plugin.render_plugin(context)
-
     try:
         content_renderer = request.toolbar.content_renderer
     except AttributeError:
