@@ -10,19 +10,20 @@ from django.templatetags.static import static
 from django.utils.safestring import mark_safe
 from django.utils.translation.trans_real import get_language, gettext
 
-from cms.utils.urlutils import static_with_version
+from cms.utils.urlutils import admin_reverse, static_with_version
 
 from . import settings as text_settings
+from .utils import cms_placeholder_add_plugin
 
 
 # this path is changed automatically whenever you run `gulp bundle`
-PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-5f73f48756.cms.ckeditor.min.js'
+PATH_TO_JS = 'djangocms_text_ckeditor/js/dist/bundle-ab7ff62c5a.cms.ckeditor.min.js'
 
 
 class TextEditorWidget(forms.Textarea):
     def __init__(self, attrs=None, installed_plugins=None, pk=None,
-                 placeholder=None, plugin_language=None, configuration=None,
-                 cancel_url=None, render_plugin_url=None, action_token=None,
+                 placeholder=None, plugin_language=None, plugin_position=None,
+                 configuration=None, cancel_url=None, render_plugin_url=None, action_token=None,
                  delete_on_cancel=False, body_css_classes=''):
         """
         Create a widget for editing text + plugins.
@@ -46,6 +47,7 @@ class TextEditorWidget(forms.Textarea):
         self.pk = pk  # specific
         self.placeholder = placeholder  # specific
         self.plugin_language = plugin_language
+        self.plugin_position = plugin_position
         if configuration and getattr(settings, configuration, False):
             conf = deepcopy(text_settings.CKEDITOR_SETTINGS)
             conf.update(getattr(settings, configuration))
@@ -104,9 +106,10 @@ class TextEditorWidget(forms.Textarea):
             'static_url': settings.STATIC_URL + 'djangocms_text_ckeditor',
             'plugin_id': self.pk,
             'plugin_language': self.plugin_language,
+            'plugin_position': self.plugin_position,
             'placeholder_id': self.placeholder.pk if self.placeholder else None,
             'render_plugin_url': self.render_plugin_url or '',
-            'add_plugin_url': self.placeholder.get_add_url() or '' if self.placeholder else '',
+            'add_plugin_url': admin_reverse(cms_placeholder_add_plugin) if self.placeholder else '',
             'clancel_plugin_url': self.cancel_url or '',
             'delete_on_cancel': self.delete_on_cancel or False,
             'action_token': self.action_token or '',
@@ -136,6 +139,7 @@ class TextEditorWidget(forms.Textarea):
             'installed_plugins': self.installed_plugins,
             'plugin_pk': self.pk,
             'plugin_language': self.plugin_language,
+            'plugin_position': self.plugin_position,
             'placeholder': self.placeholder,
             'widget': self,
             'renderer': renderer,
