@@ -12,12 +12,11 @@ from django.utils.encoding import force_str
 from django.utils.html import escape
 from django.utils.http import urlencode
 
-from cms.api import add_plugin, create_page, create_title
-from cms.models import CMSPlugin, Page, Title
+from cms.api import add_plugin, create_title
+from cms.models import CMSPlugin, Page, Placeholder
 from cms.utils.urlutils import admin_reverse
 
 from djangocms_text_ckeditor.cms_plugins import TextPlugin
-from djangocms_text_ckeditor.compat import get_page_placeholders
 from djangocms_text_ckeditor.models import Text
 from djangocms_text_ckeditor.utils import (
     _plugin_tags_to_html, _render_cms_plugin, plugin_tags_to_admin_html, plugin_tags_to_id_list, plugin_to_tag,
@@ -25,6 +24,7 @@ from djangocms_text_ckeditor.utils import (
 from tests.test_app.cms_plugins import DummyChildPlugin, DummyParentPlugin
 
 from .base import BaseTestCase
+from .fixtures import DJANGO_CMS4, DJANGOCMS_VERSIONING, TestFixture
 
 
 try:
@@ -40,7 +40,7 @@ except ImportError:
     HAS_DJANGOCMS_TRANSLATIONS = False
 
 
-class PluginActionsTestCase(BaseTestCase):
+class PluginActionsTestCase(TestFixture, BaseTestCase):
 
     def get_custom_admin_url(self, plugin_class, name):
         plugin_type = plugin_class.__name__.lower()
@@ -101,7 +101,7 @@ class PluginActionsTestCase(BaseTestCase):
 
     def _give_cms_permissions(self, user):
         for perm_type in ['add', 'change', 'delete']:
-            for model in [Page, Title]:
+            for model in [Page]:
                 self._give_permission(user, model, perm_type)
 
     def get_page_admin(self):
@@ -122,8 +122,8 @@ class PluginActionsTestCase(BaseTestCase):
         Test that you can add a text plugin
         """
         admin = self.get_superuser()
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         endpoint = self.get_add_plugin_uri(simple_placeholder, 'TextPlugin')
 
@@ -174,8 +174,8 @@ class PluginActionsTestCase(BaseTestCase):
         """
         Test that you can add a text plugin
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         endpoint = self.get_add_plugin_uri(simple_placeholder, 'TextPlugin')
 
@@ -227,8 +227,8 @@ class PluginActionsTestCase(BaseTestCase):
         Test that copy+pasting a child plugin between text editors
         creates proper copies of the child plugin and messes no other data up
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         def _get_text_plugin_with_children():
             text_plugin = add_plugin(
@@ -295,8 +295,8 @@ class PluginActionsTestCase(BaseTestCase):
         Test that you can add a text plugin
         """
         admin = self.get_superuser()
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin = add_plugin(
             simple_placeholder,
@@ -386,8 +386,8 @@ class PluginActionsTestCase(BaseTestCase):
     def test_action_token_per_session(self):
         # Assert that a cancel token for the same plugin
         # is different per user session.
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin = add_plugin(
             simple_placeholder,
@@ -409,8 +409,8 @@ class PluginActionsTestCase(BaseTestCase):
         self.assertNotEqual(action_token_1, action_token_2)
 
     def test_add_and_cancel_plugin_permissions(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         endpoint = self.get_add_plugin_uri(simple_placeholder, 'TextPlugin')
 
@@ -451,8 +451,8 @@ class PluginActionsTestCase(BaseTestCase):
         the child plugins are rendered as their contents passed
         as initial data to the text field.
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin = add_plugin(
             simple_placeholder,
@@ -494,11 +494,11 @@ class PluginActionsTestCase(BaseTestCase):
 
     def test_only_inline_editing_has_rendered_plugin_content(self):
         """
-        Tests of child plugins of a TextPlugin are rednered correctly in edit mode
+        Tests of child plugins of a TextPlugin are rendered correctly in edit mode
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
-
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
+        # import pdb; pdb.set_trace()
         text_plugin = add_plugin(
             simple_placeholder,
             'TextPlugin',
@@ -507,14 +507,35 @@ class PluginActionsTestCase(BaseTestCase):
         )
 
         self.add_plugin_to_text(text_plugin, self._add_child_plugin(text_plugin))
+        if DJANGO_CMS4:
+            try:
+                from cms.models.contentmodels import PageContent
+            except ModuleNotFoundError:
+                from cms.models.titlemodels import PageContent
+            from cms.toolbar.utils import get_object_edit_url
 
+            if DJANGOCMS_VERSIONING:
+                from djangocms_versioning.constants import DRAFT
+
+                edit_endpoint = get_object_edit_url(
+                    PageContent._original_manager.filter(
+                        page=simple_page, language='en', version__state=DRAFT
+                    ).first()
+                )
+            else:
+                try:
+                    edit_endpoint = get_object_edit_url(simple_page.get_content_obj(language='en'))
+                except AttributeError:
+                    edit_endpoint = get_object_edit_url(simple_page.get_title_obj(language='en'))
+        else:
+            edit_endpoint = simple_page.get_absolute_url()
         with self.login_user_context(self.get_superuser()):
-            response = self.client.get(simple_page.get_absolute_url() + "?edit&inline_editing=1")
+            response = self.client.get(edit_endpoint + "?edit&inline_editing=1")
             self.assertEqual(response.status_code, 200)
             self.assertContains(response, "<cms-plugin")
 
         with self.login_user_context(self.get_superuser()):
-            response = self.client.get(simple_page.get_absolute_url() + "?edit&inline_editing=0")
+            response = self.client.get(edit_endpoint + "?edit&inline_editing=0")
             self.assertEqual(response.status_code, 200)
             self.assertNotContains(response, "<cms-plugin")
 
@@ -523,8 +544,8 @@ class PluginActionsTestCase(BaseTestCase):
         No user regardless of permissions can modify the contents
         of a child plugin directly in the text plugin text.
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin = add_plugin(
             simple_placeholder,
@@ -559,8 +580,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertXMLEqual(text_plugin.body, expected_text)
 
     def test_render_child_plugin_endpoint(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = add_plugin(
             simple_placeholder,
             'TextPlugin',
@@ -568,7 +589,7 @@ class PluginActionsTestCase(BaseTestCase):
             body="I'm the first",
         )
         text_plugin_class = text_plugin.get_plugin_class_instance()
-        child_plugin = self._add_child_plugin(text_plugin)
+        child_plugin = self._add_child_plugin(text_plugin, 'LinkPlugin')
         text_plugin = self.add_plugin_to_text(text_plugin, child_plugin)
 
         with self.login_user_context(self.get_superuser()):
@@ -588,7 +609,7 @@ class PluginActionsTestCase(BaseTestCase):
                 content=rendered_content,
                 admin=True,
             )
-
+            self.maxDiff = None
             self.assertEqual(force_str(response.content), rendered_child_plugin)
 
         child_plugin = self._add_child_plugin(text_plugin, plugin_type='PreviewDisabledPlugin')
@@ -614,8 +635,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertEqual(force_str(response.content), rendered_child_plugin)
 
     def test_render_child_plugin_endpoint_calls_context_processors(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = add_plugin(
             simple_placeholder,
             'TextPlugin',
@@ -654,8 +675,8 @@ class PluginActionsTestCase(BaseTestCase):
         Users can't render a child plugin without change permissions
         on the placeholder attached object and the text plugin.
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = add_plugin(
             simple_placeholder,
             'TextPlugin',
@@ -681,8 +702,8 @@ class PluginActionsTestCase(BaseTestCase):
         was created in the current session and it's text plugin
         matches the child plugin parent.
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = add_plugin(
             simple_placeholder,
             'TextPlugin',
@@ -730,8 +751,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertEqual(force_str(response.content), 'Unable to process your request.')
 
     def test_custom_ckeditor_body_css_classes(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         parent_plugin = add_plugin(
             simple_placeholder,
@@ -760,8 +781,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertContains(response, DummyChildPlugin.child_ckeditor_body_css_class)
 
     def test_render_plugin(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = self._add_text_plugin(simple_placeholder)
 
         for i in range(0, 10):
@@ -783,8 +804,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertTrue('LinkPlugin record %d' % i in rendered)
 
     def test_render_extended_plugin(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
         text_plugin = self._add_text_plugin(simple_placeholder, 'ExtendedTextPlugin')
 
         for i in range(0, 10):
@@ -809,8 +830,8 @@ class PluginActionsTestCase(BaseTestCase):
         """
         Test that copying of textplugins replaces references to copied plugins
         """
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin = self._add_text_plugin(simple_placeholder)
 
@@ -846,7 +867,7 @@ class PluginActionsTestCase(BaseTestCase):
             'source_language': 'en',
         }
 
-        endpoint = self.get_admin_url(Page, 'copy_plugins')
+        endpoint = self.get_admin_url(Placeholder if DJANGO_CMS4 else Page, 'copy_plugins')
         endpoint += '?' + urlencode({'cms_path': '/en/'})
 
         with self.login_user_context(self.user):
@@ -855,15 +876,15 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertEqual(CMSPlugin.objects.filter(language='en').count(), 3)
             self.assertEqual(CMSPlugin.objects.filter(language=translation.language).count(), 3)
 
-            plugins = list(CMSPlugin.objects.all())
+            plugins = list(CMSPlugin.objects.order_by("id"))  # Look at the order of creation
             new_plugin = plugins[3].get_plugin_instance()[0]
             idlist = sorted(plugin_tags_to_id_list(new_plugin.body))
             expected = sorted([plugins[4].pk, plugins[5].pk])
             self.assertEqual(idlist, expected)
 
     def test_copy_plugin_callback(self):
-        simple_page = create_page('test page', 'page.html', 'en')
-        simple_placeholder = get_page_placeholders(simple_page, 'en').get(slot='content')
+        simple_page = self.create_page('test page', template='page.html', language='en')
+        simple_placeholder = self.get_placeholders(simple_page, 'en').get(slot='content')
 
         text_plugin_1 = self._add_text_plugin(simple_placeholder)
 
@@ -915,8 +936,8 @@ class PluginActionsTestCase(BaseTestCase):
             self.assertEqual(plugin_tags_to_id_list(markup), expected)
 
     def test_text_plugin_xss(self):
-        page = create_page('test page', 'page.html', 'en')
-        placeholder = get_page_placeholders(page, 'en').get(slot='content')
+        page = self.create_page('test page', template='page.html', language='en')
+        placeholder = self.get_placeholders(page, 'en').get(slot='content')
         plugin = add_plugin(placeholder, 'TextPlugin', 'en', body='body')
         endpoint = self.get_change_plugin_uri(plugin)
 
@@ -938,8 +959,8 @@ class PluginActionsTestCase(BaseTestCase):
 class DjangoCMSTranslationsIntegrationTestCase(BaseTestCase):
     def setUp(self):
         super().setUp()
-        self.page = create_page('test page', 'page.html', 'en', published=True)
-        self.placeholder = get_page_placeholders(self.page, 'en').get(slot='content')
+        self.page = self.create_page('test page', template='page.html', language='en')
+        self.placeholder = self.get_placeholders(self.page, 'en').get(slot='content')
 
     def _export_page(self):
         return json.loads(export_page(self.page, 'en'))
